//----------------------------------------------------------------------------
// 
//	file: oneneuron.cc
// 
//	function: plot dynamix property of a single neuron 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Thu Aug  6 03:25:03 CDT 1998
// 
//	$Revision: 1.1 $ $Date: 1998/10/16 10:35:21 $
//	$Log: oneneuron.cc,v $
//	Revision 1.1  1998/10/16 10:35:21  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------

#include <nnview/real.h>
#include <nnview/netlayer.h>
#include <nnview/simulog.h>
#include <nnview/real.h>
#include <nnview/plot_window.h>
#include <nnview/matrix_window.h>

// Windows
main_window     *mw;
plot_window	*aff_w;
plot_window	*lat_exc_w;
plot_window	*lat_inh_w;
plot_window	*net_input_w;
plot_window	*output_w;
plot_window	*spike_w;

// Weights
REAL	w_aff;
REAL	w_exc;
REAL	w_inh;

// Contributions
REAL	gamma_aff;
REAL	gamma_exc;
REAL	gamma_inh;

// Threshold
REAL	theta;

// Slidebars and callbacks
scrollbar *gamma_aff_sb;
scrollbar *gamma_exc_sb;
scrollbar *gamma_inh_sb;
scrollbar *theta_sb;

// others
REAL sig_low = 0.0;
REAL sig_high = 1.5;

void run();

void set_gamma_aff() {gamma_aff = gamma_aff_sb->value; }
void set_gamma_exc() {gamma_exc = gamma_exc_sb->value;}
void set_gamma_inh() {gamma_inh = gamma_inh_sb->value;}
void set_theta() {theta = theta_sb->value;}

REAL sigmoid(REAL x) {

  return (1.0/(1.0+exp(-(x-0.5))));
/*
  if (x <= sig_low)
        return 0.0;
  else if (x >= sig_high)
        return 1.0;
  else
        return (x-sig_low)/(sig_high-sig_low);
*/

}

// MAIN
int main(int argc, char *argv[]) {

  mw = new main_window(argv[1],600,800);

  aff_w = new plot_window(*mw,600,100, 0,0, 0, 10, COLOR, 100,100);
  lat_exc_w = new plot_window(*mw,600,100, 0,100, 0, 10, COLOR, 100,100);
  lat_inh_w = new plot_window(*mw,600,100, 0,200, 0, 10, COLOR, 100,100);
  net_input_w = new plot_window(*mw,600,100, 0,300, 0, 10, COLOR, 100,100);
  output_w = new plot_window(*mw,600,100, 0,400, 0, 10, COLOR, 100,100);
  spike_w = new plot_window(*mw,600,100, 0,500, 0, 10, COLOR, 100,100);
  
  callback_button *runit = new callback_button(*mw,"Run",run,
                mw->width/4,20,mw->width/2,mw->height-20); 

  new quit_button(*mw,mw->width/4,20,mw->width*3/4,mw->height-20);

  gamma_aff_sb = new scrollbar(*mw, &set_gamma_aff,
		mw->width/2,20,0,mw->height-20,
                0.0,3.0,1.0,"g_aff=%.1lf");
  gamma_exc_sb = new scrollbar(*mw, &set_gamma_exc,
		mw->width/2,20,0,mw->height-40,
                0.0,3.0,1.0,"g_exc=%.1lf");
  gamma_inh_sb = new scrollbar(*mw, &set_gamma_inh,
		mw->width/2,20,0,mw->height-60,
                0.0,3.0,1.0,"g_inh=%.1lf");
  theta_sb = new scrollbar(*mw, &set_theta,
		mw->width/2,20,0,mw->height-80,
                0.0,10.0,2.0,"theta=%.1lf");

  gamma_aff = 1.0; gamma_exc = 1.0, gamma_inh = 1.0; theta = 2.0;
  w_aff = drand48(); w_exc = drand48(); w_inh = drand48();

  mw->main_loop();

}
  

void run() {

  static REAL activity;
  REAL net_act;
  REAL spike;

  for (int i=0; i<100; ++i) {
	// activate
	net_act = gamma_aff*(1.0*w_aff) 
		+ gamma_exc*(activity*w_exc)
		- gamma_inh*(activity*w_inh);
	activity = sigmoid(net_act);

	if (activity>theta) {
		spike = 1.0;
	}

	// learn
	w_aff = (1.0-0.01)*w_aff + (0.01)*activity*activity;
	w_exc = (1.0-0.001)*w_exc + (0.001)*activity*activity;
	w_inh = (1.0-0.01)*w_inh + (0.01)*activity*activity;

	// plot
	net_input_w->push_data(net_act);
 	output_w->push_data(activity);
	aff_w->push_data(w_aff);
 	lat_exc_w->push_data(w_exc);
 	lat_inh_w->push_data(w_inh);
	spike_w->push_data(spike);

  }

}
